"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const Server_1 = require("../Server");
const services = require("../services");
const shared_1 = require("../shared");
const fs = require("fs-extra");
const path = require("path");
const tmp = require("tmp");
describe('Server Default Autokey Generated', function () {
    const PORT = 5678;
    const HOST = '127.0.0.1';
    const ProductSchema = {
        name: 'Product',
        primaryKey: 'productId',
        properties: {
            productId: { type: 'int', optional: false },
            name: { type: 'string', optional: false },
            price: { type: 'float', optional: false }
        }
    };
    let server;
    let tmpDir;
    let privateKeyPath;
    let publicKeyPath;
    before(() => __awaiter(this, void 0, void 0, function* () {
        tmpDir = tmp.dirSync();
        Realm.clearTestState();
        server = new Server_1.Server({
            port: PORT,
            dataPath: tmpDir.name,
            logLevel: 'off'
        });
        publicKeyPath = path.join(server.config.dataPath, "keys/auth.pub");
        privateKeyPath = path.join(server.config.dataPath, "keys/auth.key");
        server.addService(new services.UsernamePasswordAuthService({
            privateKeyPath: privateKeyPath,
        }));
        server.addService(new services.RealmManagementService({
            privateKeyPath: privateKeyPath,
        }));
        server.addService(new services.SyncService({
            publicKeyPath: publicKeyPath,
            dataPath: path.join(server.config.dataPath, "sync"),
        }));
        yield server.start();
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        if (server) {
            yield server.shutdown();
        }
    }));
    it('should create keys at the default location', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const publicKeyExists = yield fs.pathExists(publicKeyPath);
            const privateKeyExists = yield fs.pathExists(privateKeyPath);
            chai_1.expect(publicKeyExists).to.be.true;
            chai_1.expect(privateKeyExists).to.be.true;
        });
    });
    it('should create admin.json at the default location', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const adminJsonExists = yield fs.pathExists(path.resolve(server.config.dataPath, "keys/admin.json"));
            chai_1.expect(adminJsonExists).to.be.true;
        });
    });
    it('should be able to create an admin token user ', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const fileAsString = yield fs.readFile(path.resolve(server.config.dataPath, "keys/admin.json"), 'utf8');
            const json = JSON.parse(fileAsString);
            const adminToken = json["ADMIN_TOKEN"];
            chai_1.expect(adminToken).to.not.be.undefined;
            const user = Realm.Sync.User.adminUser(adminToken, "some url");
            const parsedToken = shared_1.parseTokenFromBase64RSA_SHA256(user.token);
            chai_1.expect(parsedToken.identity).to.eq('__admin');
        });
    });
});
//# sourceMappingURL=server-autokeygen-tests.spec.js.map