/// <reference types="express" />
/// <reference types="express-serve-static-core" />
import * as Realm from "realm";
import { Request as ExpressRequest } from 'express';
import { Discovery } from '../discovery/Discovery';
import { Server } from "../Server";
export interface AuthenticationResponseJSON {
    refresh_token: {
        token: string;
        token_data: {
            identity: string;
            is_admin: boolean;
            app_id?: string;
            access: ["refresh"];
            expires: number;
        };
    };
    authToken?: string;
}
export interface ResolvedUser {
    userId: string;
    isAdmin: boolean;
    isCreated: boolean;
}
export declare type UserMetadataRow = {
    user: User;
    key: string;
    value: string;
};
export declare type User = {
    userId: string;
    isAdmin: boolean;
    metadata?: Realm.List<UserMetadataRow>;
};
export interface AuthServiceConfiguration {
    privateKeyPath?: string;
}
export declare class AuthService {
    privateKey: string;
    private syncedRealm;
    readonly realm: Realm;
    protected discovery: Discovery;
    config: AuthServiceConfiguration;
    constructor(config?: AuthServiceConfiguration);
    authenticateOrCreateUser(body: any): Promise<ResolvedUser>;
    start(server: Server): Promise<void>;
    stop(): Promise<void>;
    authenticate(request: ExpressRequest): Promise<AuthenticationResponseJSON>;
    getUserById(userId: string, req?: Express.Request): User;
    createOrUpdateUser(userId: string, isAdmin: boolean, metadata?: {
        [key: string]: string;
    }, req?: ExpressRequest): {
        user: User;
        created: boolean;
    };
}
