"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Realm = require("realm");
const fs = require("fs-extra");
const path = require("path");
const moment = require("moment");
const superagent = require("superagent");
const URI = require("urijs");
const decorators_1 = require("../decorators");
const Server_1 = require("../Server");
const SyncedRealm_1 = require("../shared/SyncedRealm");
const shared_1 = require("../shared");
let userSchema = {
    name: 'User',
    primaryKey: 'userId',
    properties: {
        userId: { type: 'string', optional: false },
        isAdmin: { type: 'bool', optional: false },
        metadata: { type: 'list', objectType: 'UserMetadataRow', default: [], optional: false }
    }
};
let metadataSchema = {
    name: 'UserMetadataRow',
    properties: {
        user: { type: 'linkingObjects', objectType: 'User', property: 'metadata' },
        key: { type: 'string', optional: false },
        value: { type: 'string', optional: false }
    }
};
let AuthService = class AuthService {
    constructor(config = {}) {
        this.config = config;
    }
    get realm() {
        return this.syncedRealm.realm;
    }
    authenticateOrCreateUser(body) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('You must implement the authenticate method in your subclass');
        });
    }
    start(server) {
        return __awaiter(this, void 0, void 0, function* () {
            this.discovery = server.discovery;
            this.privateKey = yield fs.readFile(this.config.privateKeyPath, 'utf-8');
            this.syncedRealm = new SyncedRealm_1.SyncedRealm(this.discovery, {
                path: path.resolve(server.config.dataPath, "auth", "admin.realm"),
                schema: [userSchema, metadataSchema],
                sync: {
                    url: "realm://default/__admin",
                    user: Realm.Sync.User.adminUser(shared_1.generateAdminToken(this.privateKey), "realm://default/__admin"),
                }
            });
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.syncedRealm) {
                this.syncedRealm.close();
            }
        });
    }
    authenticate(request) {
        return __awaiter(this, void 0, void 0, function* () {
            if (request.body.path !== undefined) {
                let handle = yield this.discovery.find('realms');
                if (!handle.address || !handle.port)
                    throw new Error('Unable to contact the RealmManagement Service service named realms');
                const accessTokenUrl = new URI(`http://${handle.address}:${handle.port}/`)
                    .segment('/realms')
                    .segment('/access_token').toString();
                const response = yield superagent.post(accessTokenUrl)
                    .send(request.body);
                return response.body;
            }
            const app_id = request.body["app_id"] || 'io.realm.Auth';
            const authResponse = yield this.authenticateOrCreateUser(request.body);
            const authToken = shared_1.signToken({ user_id: authResponse.userId, is_admin: authResponse.isAdmin }, this.privateKey);
            let expires = moment().add(10, 'm').unix();
            let json = {
                refresh_token: {
                    token: authToken,
                    token_data: {
                        app_id: app_id,
                        identity: authResponse.userId,
                        is_admin: authResponse.isAdmin,
                        access: ["refresh"],
                        expires: expires
                    }
                }
            };
            return json;
        });
    }
    getUserById(userId, req) {
        const user = this.realm.objectForPrimaryKey('User', userId);
        if (!user && req) {
            throw new shared_1.JSONError({
                title: `User with userId ${userId} does not exist`,
                code: 404,
                status: 404
            });
        }
        return user;
    }
    createOrUpdateUser(userId, isAdmin, metadata = {}, req) {
        let created = this.getUserById(userId) === undefined;
        let user;
        this.realm.write(() => {
            user = this.realm.create('User', {
                userId: userId,
                isAdmin: isAdmin || false
            }, true);
            this.realm.delete(user.metadata);
            let rows = [];
            for (let key in metadata) {
                user.metadata.push({
                    user: user,
                    key: key,
                    value: metadata[key]
                });
            }
        });
        return { user, created };
    }
};
__decorate([
    decorators_1.Start(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Server_1.Server]),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "start", null);
__decorate([
    decorators_1.Stop(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "stop", null);
__decorate([
    decorators_1.Post('/'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "authenticate", null);
__decorate([
    decorators_1.Get('/users/:userId'),
    __param(0, decorators_1.Params('userId')), __param(1, decorators_1.Request()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", void 0)
], AuthService.prototype, "getUserById", null);
__decorate([
    decorators_1.Put('/users'),
    __param(0, decorators_1.Body('userId')),
    __param(1, decorators_1.Body('isAdmin')),
    __param(2, decorators_1.Body('metadata')),
    __param(3, decorators_1.Request()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Boolean, Object, Object]),
    __metadata("design:returntype", Object)
], AuthService.prototype, "createOrUpdateUser", null);
AuthService = __decorate([
    decorators_1.BaseRoute('/auth'),
    decorators_1.ServiceName('auth'),
    __metadata("design:paramtypes", [Object])
], AuthService);
exports.AuthService = AuthService;
//# sourceMappingURL=AuthService.js.map