"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const shared_1 = require("../shared");
const AuthService_1 = require("./AuthService");
const decorators_1 = require("../decorators");
const SyncedRealm_1 = require("../shared/SyncedRealm");
const shared_2 = require("../shared");
const Server_1 = require("../Server");
const crypto = require("crypto");
const path = require("path");
exports.UsernamePasswordConstants = {
    saltWorkIterations: 10000,
    keyLength: 512,
    saltLength: 256,
    digest: "sha256"
};
const PasswordSaltHashSchema = {
    name: 'Password',
    primaryKey: 'userId',
    properties: {
        userId: { type: 'string', optional: false },
        salt: { type: 'string', optional: false },
        hash: { type: 'string', optional: false },
    }
};
let UsernamePasswordAuthService = class UsernamePasswordAuthService extends AuthService_1.AuthService {
    get passwordRealm() {
        return this.syncedPasswordRealm.realm;
    }
    start(server) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            yield _super("start").call(this, server);
            this.syncedPasswordRealm = new SyncedRealm_1.SyncedRealm(this.discovery, {
                path: path.resolve(server.config.dataPath, "auth", "password.realm"),
                schema: [PasswordSaltHashSchema],
                sync: {
                    url: "realm://default/__password",
                    user: Realm.Sync.User.adminUser(shared_2.generateAdminToken(this.privateKey), "realm://default/__password"),
                }
            });
        });
    }
    changePassword(username, currentPassword, newPassword) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = this.getUserById(username);
            if (!user) {
                throw new shared_1.JSONError({
                    title: 'The user does not exist',
                    code: 404,
                    status: 404
                });
            }
            yield this.attemptToLogin(username, currentPassword);
            const salt = yield this.createSalt(exports.UsernamePasswordConstants.saltWorkIterations);
            const hash = yield this.hashPassword(newPassword, salt);
            this.passwordRealm.write(() => {
                this.passwordRealm.create('Password', {
                    salt: salt.toString("base64"),
                    hash: hash.toString("base64"),
                    userId: username
                }, true);
            });
            return {};
        });
    }
    stop() {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            yield _super("stop").call(this);
            if (this.syncedPasswordRealm) {
                this.syncedPasswordRealm.close();
            }
        });
    }
    hashPassword(clearTextPassword, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            return shared_1.Promisify(crypto.pbkdf2.bind(crypto), clearTextPassword, salt, exports.UsernamePasswordConstants.saltWorkIterations, exports.UsernamePasswordConstants.keyLength, exports.UsernamePasswordConstants.digest);
        });
    }
    createSalt(saltWorkFactor) {
        return __awaiter(this, void 0, void 0, function* () {
            return shared_1.Promisify(crypto.randomBytes.bind(crypto), saltWorkFactor);
        });
    }
    comparePassword(candidatePassword, salt, hash) {
        return __awaiter(this, void 0, void 0, function* () {
            const hashedCandidatePassword = yield this.hashPassword(candidatePassword, salt);
            return hashedCandidatePassword.equals(hash);
        });
    }
    attemptToRegister(username, password, isAdmin) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            const foundUser = _super("getUserById").call(this, username);
            if (foundUser) {
                throw new shared_1.JSONError({
                    detail: `Given account: ${username}`,
                    title: 'The provided credentials are invalid or a user already exists.',
                    code: 611
                });
            }
            const salt = yield this.createSalt(exports.UsernamePasswordConstants.saltWorkIterations);
            const hash = yield this.hashPassword(password, salt);
            this.passwordRealm.write(() => {
                this.passwordRealm.create('Password', {
                    salt: salt.toString("base64"),
                    hash: hash.toString("base64"),
                    userId: username
                }, true);
            });
            const { user, created } = _super("createOrUpdateUser").call(this, username, isAdmin ? true : false);
            return {
                userId: user.userId,
                isAdmin: user.isAdmin,
                isCreated: created
            };
        });
    }
    attemptToLogin(username, password) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            const foundUser = _super("getUserById").call(this, username);
            if (!foundUser) {
                throw new shared_1.JSONError({
                    detail: `Given account: ${username}`,
                    title: 'The provided credentials are invalid or a user does not exist',
                    status: 400,
                    code: 611
                });
            }
            const foundPassword = this.passwordRealm.objectForPrimaryKey('Password', username);
            const passwordMatch = yield this.comparePassword(password, new Buffer(foundPassword.salt, "base64"), new Buffer(foundPassword.hash, "base64"));
            if (!passwordMatch) {
                throw new shared_1.JSONError({
                    detail: `Given account: ${username}`,
                    status: 400,
                    code: 611,
                    title: 'The provided credentials are invalid or a user does not exist'
                });
            }
            return {
                userId: foundUser.userId,
                isAdmin: foundUser.isAdmin,
                isCreated: false
            };
        });
    }
    authenticateOrCreateUser(body) {
        return __awaiter(this, void 0, void 0, function* () {
            let username = "";
            let password = "";
            let register = false;
            let isAdmin = undefined;
            if (body["user_info"]) {
                username = body["data"];
                password = body["user_info"]["password"];
                register = body["user_info"]["register"];
                isAdmin = body["user_info"]["is_admin"] || false;
            }
            else {
                username = body["username"];
                password = body["password"];
                register = body["register"] || false;
                isAdmin = body["is_admin"] || false;
            }
            if (!username) {
                throw new shared_1.JSONError({
                    title: "Your request did not validate because of missing parameter 'username' .",
                    code: 602,
                    path: 'missing-parameters'
                });
            }
            if (password === undefined || password === null) {
                throw new shared_1.JSONError({
                    title: "Your request did not validate because of missing parameter 'password'",
                    code: 602,
                    path: 'missing-parameters'
                });
            }
            if (register) {
                return this.attemptToRegister(username, password);
            }
            else {
                return this.attemptToLogin(username, password);
            }
        });
    }
};
__decorate([
    decorators_1.Start(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Server_1.Server]),
    __metadata("design:returntype", Promise)
], UsernamePasswordAuthService.prototype, "start", null);
__decorate([
    decorators_1.Put('/users/:username/password'),
    __param(0, decorators_1.Params('username')),
    __param(1, decorators_1.Body('currentPassword')),
    __param(2, decorators_1.Body('newPassword')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String]),
    __metadata("design:returntype", Promise)
], UsernamePasswordAuthService.prototype, "changePassword", null);
__decorate([
    decorators_1.Stop(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], UsernamePasswordAuthService.prototype, "stop", null);
UsernamePasswordAuthService = __decorate([
    decorators_1.BaseRoute('/auth'),
    decorators_1.ServiceName('auth')
], UsernamePasswordAuthService);
exports.UsernamePasswordAuthService = UsernamePasswordAuthService;
//# sourceMappingURL=UsernamePasswordAuthService.js.map